@@ -120,6 +120,8 @@ async def _error_wrapper(func, *, args=(), kwargs=None, retries):
             err = e
             if "SlowDown" in str(e):
                 await asyncio.sleep(min(1.7**i * 0.1, 15))
+            elif "XAmzContentSHA256Mismatch" in str(e):
+                await asyncio.sleep(min(1.7**i * 0.1, 15))
             else:
                 break
         except Exception as e:
@@ -235,6 +237,14 @@ class S3FileSystem(AsyncFileSystem):
     session : aiobotocore AioSession object to be used for all connections.
          This session will be used inplace of creating a new session inside S3FileSystem.
          For example: aiobotocore.session.AioSession(profile='test_user')
+    max_concurrency : int (1)
+        The maximum number of concurrent transfers to use per file for multipart
+        upload (``put()``) operations. Defaults to 1 (sequential). When used in
+        conjunction with ``S3FileSystem.put(batch_size=...)`` the maximum number of
+        simultaneous connections is ``max_concurrency * batch_size``. We may extend
+        this parameter to affect ``pipe()``, ``cat()`` and ``get()``. Increasing this
+        value will result in higher memory usage during multipart upload operations (by
+        ``max_concurrency * chunksize`` bytes per file).
 
     The following parameters are passed on to fsspec:
 
@@ -282,6 +292,7 @@ class S3FileSystem(AsyncFileSystem):
         cache_regions=False,
         asynchronous=False,
         loop=None,
+        max_concurrency=1,
         **kwargs,
     ):
         if key and username:
@@ -319,6 +330,9 @@ class S3FileSystem(AsyncFileSystem):
         self.cache_regions = cache_regions
         self._s3 = None
         self.session = session
+        if max_concurrency < 1:
+            raise ValueError("max_concurrency must be >= 1")
+        self.max_concurrency = max_concurrency
 
     @property
     def s3(self):
@@ -784,7 +798,9 @@ class S3FileSystem(AsyncFileSystem):
             raise ValueError("Cannot traverse all of S3")
         return await super()._glob(path, **kwargs)
 
-    async def _find(self, path, maxdepth=None, withdirs=None, detail=False, prefix=""):
+    async def _find(
+        self, path, maxdepth=None, withdirs=None, detail=False, prefix="", **kwargs
+    ):
         """List all files below path.
         Like posix ``find`` command without conditions
 
@@ -811,7 +827,11 @@ class S3FileSystem(AsyncFileSystem):
             )
         if maxdepth:
             return await super()._find(
-                bucket + "/" + key, maxdepth=maxdepth, withdirs=withdirs, detail=detail
+                bucket + "/" + key,
+                maxdepth=maxdepth,
+                withdirs=withdirs,
+                detail=detail,
+                **kwargs,
             )
         # TODO: implement find from dircache, if all listings are present
         # if refresh is False:
@@ -822,7 +842,7 @@ class S3FileSystem(AsyncFileSystem):
         #     elif len(out) == 0:
         #         return super().find(path)
         #     # else: we refresh anyway, having at least two missing trees
-        out = await self._lsdir(path, delimiter="", prefix=prefix)
+        out = await self._lsdir(path, delimiter="", prefix=prefix, **kwargs)
         if not out and key:
             try:
                 out = [await self._info(path)]
@@ -984,9 +1004,12 @@ class S3FileSystem(AsyncFileSystem):
         else:
             files = await self._lsdir(path, refresh, versions=versions)
             if not files and "/" in path:
-                files = await self._lsdir(
-                    self._parent(path), refresh=refresh, versions=versions
-                )
+                try:
+                    files = await self._lsdir(
+                        self._parent(path), refresh=refresh, versions=versions
+                    )
+                except IOError:
+                    pass
                 files = [
                     o
                     for o in files
@@ -1115,6 +1138,7 @@ class S3FileSystem(AsyncFileSystem):
                 "create_multipart_upload", Bucket=bucket, Key=key, **kwargs
             )
 
+            # TODO: cancel MPU if the following fails
             out = [
                 await self._call_s3(
                     "upload_part",
@@ -1140,7 +1164,13 @@ class S3FileSystem(AsyncFileSystem):
         self.invalidate_cache(path)
 
     async def _put_file(
-        self, lpath, rpath, callback=_DEFAULT_CALLBACK, chunksize=50 * 2**20, **kwargs
+        self,
+        lpath,
+        rpath,
+        callback=_DEFAULT_CALLBACK,
+        chunksize=50 * 2**20,
+        max_concurrency=None,
+        **kwargs,
     ):
         bucket, key, _ = self.split_path(rpath)
         if os.path.isdir(lpath):
@@ -1169,24 +1199,15 @@ class S3FileSystem(AsyncFileSystem):
                 mpu = await self._call_s3(
                     "create_multipart_upload", Bucket=bucket, Key=key, **kwargs
                 )
-
-                out = []
-                while True:
-                    chunk = f0.read(chunksize)
-                    if not chunk:
-                        break
-                    out.append(
-                        await self._call_s3(
-                            "upload_part",
-                            Bucket=bucket,
-                            PartNumber=len(out) + 1,
-                            UploadId=mpu["UploadId"],
-                            Body=chunk,
-                            Key=key,
-                        )
-                    )
-                    callback.relative_update(len(chunk))
-
+                out = await self._upload_file_part_concurrent(
+                    bucket,
+                    key,
+                    mpu,
+                    f0,
+                    callback=callback,
+                    chunksize=chunksize,
+                    max_concurrency=max_concurrency,
+                )
                 parts = [
                     {"PartNumber": i + 1, "ETag": o["ETag"]} for i, o in enumerate(out)
                 ]
@@ -1201,6 +1222,54 @@ class S3FileSystem(AsyncFileSystem):
             self.invalidate_cache(rpath)
             rpath = self._parent(rpath)
 
+    async def _upload_file_part_concurrent(
+        self,
+        bucket,
+        key,
+        mpu,
+        f0,
+        callback=_DEFAULT_CALLBACK,
+        chunksize=50 * 2**20,
+        max_concurrency=None,
+    ):
+        max_concurrency = max_concurrency or self.max_concurrency
+        if max_concurrency < 1:
+            raise ValueError("max_concurrency must be >= 1")
+
+        async def _upload_chunk(chunk, part_number):
+            result = await self._call_s3(
+                "upload_part",
+                Bucket=bucket,
+                PartNumber=part_number,
+                UploadId=mpu["UploadId"],
+                Body=chunk,
+                Key=key,
+            )
+            callback.relative_update(len(chunk))
+            return result
+
+        out = []
+        while True:
+            chunks = []
+            for i in range(max_concurrency):
+                chunk = f0.read(chunksize)
+                if chunk:
+                    chunks.append(chunk)
+            if not chunks:
+                break
+            if len(chunks) > 1:
+                out.extend(
+                    await asyncio.gather(
+                        *[
+                            _upload_chunk(chunk, len(out) + i)
+                            for i, chunk in enumerate(chunks, 1)
+                        ]
+                    )
+                )
+            else:
+                out.append(await _upload_chunk(chunk, len(out) + 1))
+        return out
+
     async def _get_file(
         self, rpath, lpath, callback=_DEFAULT_CALLBACK, version_id=None
     ):
@@ -1382,7 +1451,7 @@ class S3FileSystem(AsyncFileSystem):
         path = self._strip_protocol(path).strip("/")
         # Send buckets to super
         if "/" not in path:
-            if path in await self._lsbuckets(path):
+            if path == "":
                 return True
             try:
                 out = await self._lsdir(path)
